################################ import.tcl #################################
# This module contains code used to import email message into the
# email database from other sources.
#

# Import email messages from another email database.  Only import those
# messages whose msg_uuid (corresponding to Message-ID: in the header)
# is not already found in our local msg_uuid header.  By only importing
# new messages this way, we can "import" from other database in order
# to bring two or more database into alignment.
#
proc import_from_db {} {
  set types {{{All Files} *}}
  set file [tk_getOpenFile -filetypes $types]
  if {$file==""} return
  ::mailroom::db eval "attach '[string map {' ''} $file]' as aux"
  set rc [catch _import_from_aux msg]
  ::mailroom::db eval "detach aux"
  if {$rc} {error $msg}
}
proc _import_from_aux {} {
  show_msg
  .nb.p0.t delete 1.0 end
  ::mailroom::db transaction {
    ::mailroom::db eval {
       SELECT priority, data, rcvd, aux.msg_uuid.msgid AS uuid
         FROM aux.msg, aux.blob, aux.msg_uuid
        WHERE aux.blob.blobid=aux.msg.msgid
          AND aux.msg_uuid.msgid=aux.msg.msgid
          AND aux.msg_uuid.uuid IN (
               SELECT uuid FROM aux.msg_uuid
               EXCEPT SELECT uuid FROM main.msg_uuid)
    } {
      if {[string is integer $priority]} {
        set priority [lindex {spam trash old old inbox unread sent unsent} $priority]
      }
      if {$priority==""} {set priority inbox}
      if {$priority=="trash" || $priority=="spam"} continue
      ::mailroom::message insert -string $data -bucket $priority -receivedTime $rcvd
      .nb.p0.t insert end $uuid\n
      .nb.p0.t see end
      update
    }
  }
  message_rebuild_catalog_when_idle
}

# Save the raw text of a single email message to the file descriptor
#
proc export_raw {out msgid} {
  ::mailroom::db eval {
    SELECT data FROM msg JOIN blob ON blobid=msgid WHERE msgid=$msgid
  } break
  if {![info exists data]} return
  if {[regexp "^X-From: (\[^\n]*\n)" $data all header]} {
    set n [string length $all]
    puts -nonewline $out "From $header[string range $data $n end]"
    return
  }
  set source {unknown}
  set date [clock format [clock seconds]]
  regexp "Return-Path: <(\[^>\n\r\]+)>" $data all source
  regexp "Date: (\[^\n\r\]+)" $data all date
  puts -nonewline $out "From $source $date\n$data"
}

# Prompt the user for the name of a file, then append the raw text
# of the currently selected message into that file.
#
proc export {type} {
  set types {{{All Files} *}}
  set file [tk_getSaveFile -filetypes $types]
  if {$file==""} return
  set out [open $file a]
  fconfigure $out -translation binary
  if {$type=="selected"} {
    global selected
    export_raw $out $selected
  } elseif {$type=="visible"} {
    foreach msgid [.nb.p1.t children {}] {
      export_raw $out $msgid
      update
    }
  } elseif {$type=="all"} {
    ::mailroom::db eval {SELECT msgid FROM msg} {
      export_raw $out $msgid
      update
    }
  }
  close $out
}
